function print_cv_results_latex(setting, varargin)
	% Print a table with cross validation results
	
	if nargin < 1
		setting = '5fold-cols';
	end
	
	opts = struct(varargin{:});
	if isfield(opts,'filename')
		file = fopen(opts.filename, 'wt');
		opened_file = 1;
	else
		file = stdout;
		opened_file = 0;
	end
	
	% Options
	opts.include_stddev = 1;
	opts.include_best_value = 1;
	opts.highlight_best = 1;
	opts.highlight_significant = 1;
	opts.significance_threshold = 0.05;
	opts.include_split_unique = 0;
	opts.highlight_best_split = 0;
	
	datasets = {};
	datasets{end+1} = 'e';
	datasets{end+1} = 'ic';
	datasets{end+1} = 'gpcr';
	datasets{end+1} = 'nr';
	
	methods = {};
	if isequal(setting,'baseline-loo')
		opts.include_stddev = 0;
		opts.include_best_value = 0;
		opts.include_split_unique = 1;
		methods{end+1} = struct('name','adjacent', 'caption','Baseline');
		methods{end+1} = struct('name','adjacent-cheat', 'caption','Cheating');
		setting = 'loo-pairs'
	elseif isequal(setting,'baseline')
		% For "Validation of machine learning methods for drug–target interaction can have severe biases" (2013)
		opts.include_stddev = 0;
		opts.include_best_value = 0;
		opts.include_split_unique = 1;
		%opts.highlight_best = 0;
		%opts.highlight_significant = 0;
		if ~isfield(opts,'include_slight_cheat')
			opts.include_slight_cheat = ~isempty(strfind(opts.setting,'fold'));
		end
		
		methods{end+1} = struct('name','adjacent', 'caption','Baseline');
		if opts.include_slight_cheat
			methods{end+1} = struct('name','adjacent-slighter-cheat', 'caption','Slight cheat');
		end
		methods{end+1} = struct('name','adjacent-cheat', 'caption','Cheating');
		
		setting = opts.setting;
	else
		error('unknown setting: %s',setting);
	end
	
	dont_need_significance = {'num_bootstraps',0};
	
	if opts.include_split_unique
		fprintf(file,'\\begin{tabular}{ll@{\\hspace*{6mm}}ccc@{\\hspace*{6mm}}ccc}');
	else
		fprintf(file,'\\begin{tabular}{l@{\\hspace{2mm}}l@{\\hspace{2mm}}l@{\\hspace{2mm}}l');
		if opts.include_best_value
			fprintf(file,'@{\\hspace{2mm}}l}');
		end
	end
	fprintf(file,'\n');
	fprintf(file,'\\hline\n');
	fprintf(file,'\\noalign{\\smallskip}\n');
	if opts.include_split_unique
		fprintf(file,'        &        & \\multicolumn{3}{c}{AUC} & \\multicolumn{3}{c}{AUPR} \\\\\n');
		fprintf(file,'Dataset & Method & overall & unique & other & overall & unique  & other');
	elseif opts.include_stddev
		fprintf(file,'Dataset & Method & AUC (std) &  AUPR (std)');
	else
		fprintf(file,'Dataset & Method & AUC &  AUPR');
	end
	if opts.include_best_value
		fprintf(file,' & $T$ (std)');
	end
	fprintf(file,'\\\\\n');
	
	for id=1:numel(datasets)
		results = cell(1,numel(methods));
		dummies = cell(1,numel(methods));
		for im=1:numel(methods)
			if isfield(methods{im},'dummy_auc')
				% dummy results, don't include
				results{im} = results{im-1};
				dummies{im}.auc  = getfield(methods{im}.dummy_auc, datasets{id});
				dummies{im}.aupr = getfield(methods{im}.dummy_aupr,datasets{id});
			else
				data = load(['results/cv-',setting,'-',datasets{id},'-',methods{im}.name,'.mat']);
				results{im} = data.results;
			end
		end
		
		stats = stats_for_multiple_cross_validates(results, 'dummies',{dummies}, varargin{:});
		for im=1:numel(methods)
			if isfield(methods{im},'dummy_auc')
				%stats.auc_mean(im) = getfield(methods{im}.dummy_auc,datasets{id});
			end
			if isfield(methods{im},'dummy_aupr')
				dummy.aupr_mean = getfield(methods{im}.dummy_aupr,datasets{id});
				lstats{im} = dummy;
			else
				lstats{im} = stats_for_cross_validate(results{im}, varargin{:});
			end
		end
		
		if opts.include_split_unique
			data = load_dataset(results{1}.dataset_name);
			is_unique = bsxfun(@or, sum(data.y,1)==1, sum(data.y,2)==1);
			in_u = (data.y & is_unique)  | ~data.y;
			in_n = (data.y & ~is_unique) | ~data.y;
			for im=1:numel(methods)
				r = results{im};
				s_u = cell(1,numel(r.predictions));
				s_n = cell(1,numel(r.predictions));
				for i=1:numel(r.predictions)
					s_u{i} = r.predictions{i}(in_u);
					s_n{i} = r.predictions{i}(in_n);
				end
				scores_u{im} = s_u;
				scores_n{im} = s_n;
			end
			stats_u = stats_for_multiple_predictions({data.y(in_u)}, scores_u, varargin{:}, dont_need_significance{:});
			stats_n = stats_for_multiple_predictions({data.y(in_n)}, scores_n, varargin{:}, dont_need_significance{:});
			for im=1:numel(methods)
				lstats_u{im} = stats_for_predictions({data.y(in_u)}, scores_u{im}, dont_need_significance{:});
				lstats_n{im} = stats_for_predictions({data.y(in_n)}, scores_n{im}, dont_need_significance{:});
			end
		end
		
		auprs = cellfun(@(x)x.aupr_mean,lstats);
		[best_auc_value,best_auc] = max(stats.auc_mean);
		[best_aupr_value,best_aupr] = max(auprs);
		
		fprintf(file,'\\noalign{\\smallskip}');
		fprintf(file,'\\hline');
		fprintf(file,'\\noalign{\\smallskip}\n');
		
		%stats.auc_pvalue
		
		fprintf(file,'\\multirow{%d}{*}{\\%s}\n',numel(methods),datasets{id});
		for im=1:numel(methods)
			fprintf(file,' & %-10s', methods{im}.caption);
			
			% auc
			fprintf(file,' & ');
			if opts.highlight_best && stats.auc_mean(im) >= best_auc_value
				fprintf(file,'\\best');
			elseif opts.highlight_significant
				if stats.auc_pvalue(im,best_auc) >= opts.significance_threshold
					fprintf(file,'\\insignificant');
				else
					fprintf(file,'\\significant');
				end
				fprintf(file,'[%.4f,%.4f]', stats.auc_pvalue(im,best_auc), stats.bootstrap_p_auc(im,best_auc));
			end
			fprintf(file,'{%.3f}', stats.auc_mean(im));
			if opts.include_stddev
				fprintf(file,' \\stddev{%.3f}', sqrt(stats.auc_cov(im,im)));
			end
			
			if opts.include_split_unique
				fprintf(file,' & ');
				if opts.highlight_best_split && stats_u.auc_mean(im) >= max(stats_u.auc_mean)
					fprintf(file,'\\best');
				end
				fprintf(file,'{%.3f}', stats_u.auc_mean(im));
				
				fprintf(file,' & ');
				if opts.highlight_best_split && stats_n.auc_mean(im) >= max(stats_n.auc_mean)
					fprintf(file,'\\best');
				end
				fprintf(file,'{%.3f}', stats_n.auc_mean(im));
				
			end
			
			% aupr
			fprintf(file,' & ');
			if opts.highlight_best && lstats{im}.aupr_mean >= best_aupr_value
				fprintf(file,'\\best');
			elseif opts.highlight_significant
				if stats.bootstrap_p_aupr(im,best_aupr) >= opts.significance_threshold
					fprintf(file,'\\insignificant');
				else
					fprintf(file,'\\significant');
				end
				fprintf(file,'[%.4f]', stats.bootstrap_p_aupr(im,best_aupr));
			end
			fprintf(file,'{%.3f}', lstats{im}.aupr_mean);
			if opts.include_stddev
				fprintf(file,' \\stddev{%.3f}', stats.bootstrap_std_aupr(im));
			end
			
			if opts.include_split_unique
				fprintf(file,' & ');
				if opts.highlight_best_split && lstats_u{im}.aupr_mean >= max(cellfun(@(x)x.aupr_mean,lstats_u))
					fprintf(file,'\\best');
				end
				fprintf(file,'{%.3f}', lstats_u{im}.aupr_mean);
				
				fprintf(file,' & ');
				if opts.highlight_best_split && lstats_n{im}.aupr_mean >= max(cellfun(@(x)x.aupr_mean,lstats_n))
					fprintf(file,'\\best');
				end
				fprintf(file,'{%.3f}', lstats_n{im}.aupr_mean);
			end
			
			% best_value, i.e. decay rate
			if opts.include_best_value && isfield(results{im}.logs{1},'best_value');
				vals = cellfun(@(x)x.best_value, results{im}.logs(:));
				fprintf(file,' & {%.3f \\stddev{%.3f}}', mean(vals), std(vals));
			end
			
			fprintf(file,'\\\\\n');
		end
	end
	
	fprintf(file,'\\noalign{\\smallskip}');
	fprintf(file,'\\hline\n');
	fprintf(file,'\\end{tabular}\n');
	
	if opened_file
		fclose(file);
	end
end
