
function results = run_cross_validate(fun, data, varargin)
	% Run cross-validation test for interaction prediction.
	% Does not calculate statistics of the results, use single_run_stats() for that
	%
	% Usage:
	%   results = run_cross_validate(fun, data [,options])
	% 
	% fun:  handle to the link prediction function to test
	% data: either a dataset structure as returned by load_dataset, or the name of a dataset.
	% mode: (optional) testing mode. Possible values:
	%         - 'kernel':   (default) Use the kernels from the dataset
	%         - 'nokernel': Use the identity matrix instead of the kernels of the dataset
	%         - 'ideal':    Use ideal kernels, i.e. y*y'
	% options: more options, in the form of 'key',value pairs
	%    'kernel',mode        use the given kernel
	%    'cv_type',type       type of cross-validation: {'pairs','rows','cols'}
	%    'loo',1:             use leave one out cross-validation instead of 10-fold cv
	%    'num_folds',10:      number of folds for cv
	%    'num_repetitions',5: number of repetitions of the experiment
	%    'missing',X:         use X for missing values instead of 0
	%    'negatives_missing',X: treat negative values as missing
	% 
	
	if nargin < 2
		help run_cross_validate;
		return;
	end
	
	% options
	opts = struct(varargin{:});
	if ~isfield(opts,'kernel'), opts.kernel = 'given'; end
	
	% Load data / function by name
	if ~isstruct(data)
		data = load_dataset(data);
	end
	if ischar(fun)
		fun = method_by_name(fun, varargin{:});
	end
	
	% which kernels to use?
	if isequal(opts.kernel,'given') && isfield(data,'k1') && isfield(data,'k2')
		k1 = data.k1;
		k2 = data.k2;
	elseif isequal(opts.kernel,'eye')
		k1 = eye(size(data.y,1));
		k2 = eye(size(data.y,2));
	elseif isequal(opts.kernel,'kernel1') && isfield(data,'k1')
		k1 = data.k1;
		k2 = eye(size(data.y,2));
	elseif isequal(opts.kernel,'kernel2') && isfield(data,'k2')
		k1 = eye(size(data.y,1));
		k2 = data.k2;
	elseif isequal(opts.kernel,'ideal')
		k1 = ideal_kernel(data.y);
		k2 = ideal_kernel(data.y');
	elseif isequal(opts.kernel,'random')
		k1 = random_kernel(size(data.y,1));
		k2 = random_kernel(size(data.y,2));
	else
		error(['unknown kernel mode: ' opts.kernel]);
	end
	
	results = do_run_cross_validate(fun, data.y, k1, k2, varargin{:});
	results.dataset_name = data.name;
end

function k=ideal_kernel(y)
	% Ideal kernel is the outer product of the target y values we want to predict
	k=y*y';
	d=diag(diag(k).^-0.5);
	k=d*k*d + 0.001*eye(size(k));
end
function k=random_kernel(n)
	r=rand(n,n);
	k=r*r';
end

function results = do_run_cross_validate(fun, y,k1,k2, varargin)
	% Set the random seed, so we get exactly the same folds every time
	rand('state',1234567890);
	
	% Parse arguments
	opts = struct(varargin{:});
	% leave out entire rows, instead of doing CV per instance
	if ~isfield(opts,'cv_type'), opts.cv_type = 'pairs'; end
	% negatives as missing?
	if ~isfield(opts,'negatives_missing'), opts.negatives_missing = isequal(opts.cv_type,'pairs'); end
	% label missing values as ...
	if ~isfield(opts,'missing'), opts.missing = 0; end
	% Combine negatives into one fold? (only for leave one out)
	if ~isfield(opts,'combine_negatives'), opts.combine_negatives = true; end
	
	if isfield(opts,'num_folds') && isequal(opts.num_folds, 'loo')
		% Leave one out
		opts.loo = 1;
		if isequal(opts.cv_type,'rows')
			opts.num_folds = size(y,1);
		elseif isequal(opts.cv_type,'cols')
			opts.num_folds = size(y,2);
		else
			opts.num_folds = numel(y);
		end
		if ~isfield(opts,'num_repetitions'), opts.num_repetitions = 1; end
	else
		% k-fold cross validation
		opts.loo = 0;
		if ~isfield(opts,'num_folds'), opts.num_folds = 10; end
		if ~isfield(opts,'num_repetitions'), opts.num_repetitions = 10; end
		if ischar(opts.num_folds) % parse '10fold' as 10
			[a,b] = sscanf(opts.num_folds,'%d');
			opts.num_folds = a;
			if isempty(opts.num_folds), error('num_folds should not be empty'); end;
		end
	end
	
	% Split data into folds
	[na,nb] = size(y);
	if isequal(opts.cv_type, 'rows')
		division = repmat(1:opts.num_folds,1,ceil(na/opts.num_folds));
		for rep = 1:opts.num_repetitions
			rows = division(randperm(na));
			rows = reshape(rows,na,1);
			which_fold{rep} = repmat(rows,1,nb); % matlab is column major
		end
	elseif isequal(opts.cv_type, 'cols')
		division = repmat(1:opts.num_folds,1,ceil(nb/opts.num_folds));
		for rep = 1:opts.num_repetitions
			cols = division(randperm(nb));
			cols = reshape(cols,1,nb);
			which_fold{rep} = repmat(cols,na,1);
		end
	elseif isequal(opts.cv_type, 'pairs')
		division = repmat(1:opts.num_folds,1,ceil(na*nb/opts.num_folds)); % = [1,2,3,1,2,3,1,2,3,...]
		for rep = 1:opts.num_repetitions
			which_fold{rep} = division(randperm(na*nb));
		end
	else
		error('Unknown cross validate type (cv_type = %s)',opts.cv_type);
	end
	% TODO: divide the positives and negatives separately?

	% Optimization: with leave one out we can test all negatives at once, since leaving them out does nothing
	% If you don't trust me, set this variable to 0 and observe that the result stays the same
	i_trust_this_reasoning = 1;
	if i_trust_this_reasoning && opts.combine_negatives && opts.loo && isequal(opts.cv_type,'pairs') && opts.missing == 0
		% Leave one out
		opts.num_folds = nnz(y)+1;
		which = zeros(size(y));
		which(find(y)) = 1:nnz(y);
		which(~y) = nnz(y)+1; % do the negatives in fold nnz(y)+1
		for rep = 1:opts.num_repetitions
			which_fold{rep} = which;
		end
	end
	
	% Run the tests
	predictions = cell(opts.num_repetitions, 1);
	logs = cell(opts.num_repetitions, opts.num_folds);
	for rep = 1:opts.num_repetitions
		predictions{rep} = zeros(na,nb);
		for fold = 1:opts.num_folds
			% Set values outside this fold to 0/opts.missing
			which = (which_fold{rep}==fold);
			y_fold = y;
			y_fold(which) = opts.missing;
			% Which are the values to predict?
			missing = reshape(which, size(y));
			if opts.negatives_missing
				missing = missing | (y == 0);
			end
			% Predict
			tic;
			[prediction_fold, log_fold] = fun(y_fold,k1,k2,missing);
			time = toc;
			assert(isequal(size(prediction_fold), size(y)));
			% Store
			predictions{rep}(which) = prediction_fold(which);
			log_fold.runtime = time;
			logs{rep,fold} = log_fold;
		end
	end
	
	% Done
	results = opts;
	results.which_fold = which_fold; % Which entry was tested in which fold (for each repetition)
	results.predictions = predictions; % Score for each entry (for each repetition)
	results.logs = logs;
	results.fun_name = function_to_string(fun);
end
