
function stats = stats_for_multiple_predictions(actuals, scores, varargin)
	% Covariance matrix of AUC as per
	% Comparing the Areas under Two or More Correlated Receiver Operating Characteristic Curves: A Nonparametric Approach
	% Elizabeth R. DeLong, David M. DeLong and Daniel L. Clarke-Pearson
	
	opts = struct(varargin{:});
	if ~isfield(opts,'num_bootstraps') opts.num_bootstraps = 5000; end
	
	stats = struct();
	num_scores = numel(scores);
	[num_repetitions,num_parts] = size(actuals);
	
	if num_parts ~= 1
		error('num_parts should be 1');
	end
	
	sorted_actuals = cell(num_scores,num_parts*num_repetitions);
	for j = 1:num_scores
		for i = 1:numel(actuals)
			[unused,order] = sort(scores{j}{i}(:),'descend');
			sorted_actuals{j,i} = actuals{i}(:)(order);
		end
	end
	
	%% Weight of the parts
	%part_weight = cellfun(@numel,actuals);
	%part_weight = part_weight / sum(part_weight(1,:));
	
	% Brute force calculation?
	
	% ψ(X,Y) = 1   if Y < X
	%          1/2 if Y = X
	%          0   if Y > X
	%        = gt(X,Y) + eq(X,Y)/2
	
	% AUC = expect value of ψ(score of postives,score of negatives)
	
	% ξ₁₀ = E[ ψ(X,Y)ψ(X,Y') ] for different Y and Y'?
	% ξ₁₁ = E[ ψ(X,Y)² ]
	
	M = zeros(num_scores, num_repetitions);
	for is = 1:num_scores
		for ir = 1:num_repetitions
			% Entry of mean AUC matrix
			neg = scores{is}{ir}(actuals{ir} == 0)(:);
			pos = scores{is}{ir}(actuals{ir} == 1)(:);
			%phis = bsxfun(@gt,pos,neg') + bsxfun(@eq,pos,neg')*0.5;
			%M(is,ir) = mean(phis(:));
			M(is,ir) = sum(with_lteqgt(pos,neg, @(l,e,g) g+0.5*e)) / (numel(pos) * numel(neg));
		end
	end
	stats.aucs_mean = M;
	stats.auc_mean = mean(M,2);
	
	for is1 = 1:num_scores
		for is2 = 1:num_scores
			for ir1 = 1:num_repetitions
				for ir2 = 1:num_repetitions
					% Entry of covariance matrix
					neg1 = scores{is1}{ir1}(actuals{ir1} == 0)(:);
					neg2 = scores{is2}{ir2}(actuals{ir2} == 0)(:);
					pos1 = scores{is1}{ir1}(actuals{ir1} == 1)(:);
					pos2 = scores{is2}{ir2}(actuals{ir2} == 1)(:);
					%phis1 = bsxfun(@gt,pos1,neg1') + bsxfun(@eq,pos1,neg1')*0.5 - M(is1,ir1);
					%phis2 = bsxfun(@gt,pos2,neg2') + bsxfun(@eq,pos2,neg2')*0.5 - M(is2,ir2);
					%[m,n] = size(phis1);
					%s10 = sum( mean(phis1,2) .* mean(phis2,2) ) / (m-1);
					%s01 = sum( mean(phis1,1) .* mean(phis2,1) ) / (n-1);
					%S(ir1,ir2,is1,is2) = s10/m + s01/n;
					mphis1n = with_lteqgt(pos1,neg1, @(l,e,g)g+0.5*e) / numel(neg1) - M(is1,ir1);
					mphis2n = with_lteqgt(pos2,neg2, @(l,e,g)g+0.5*e) / numel(neg2) - M(is2,ir2);
					mphis1p = with_lteqgt(neg1,pos1, @(g,e,l)g+0.5*e) / numel(pos1) - M(is1,ir1);
					mphis2p = with_lteqgt(neg2,pos2, @(g,e,l)g+0.5*e) / numel(pos2) - M(is2,ir2);
					s10 = sum( mphis1n .* mphis2n ) / (numel(pos1)-1);
					s01 = sum( mphis1p .* mphis2p ) / (numel(neg1)-1);
					S(ir1,ir2,is1,is2) = s10/numel(pos1) + s01/numel(neg1);
				end
			end
		end
	end
	stats.aucs_cov = S;
	stats.auc_cov = squeeze(sum(sum(S,2),1)) / num_repetitions^2;
	
	% Dummy values
	if isfield(opts,'dummies')
		for is=1:num_scores
			if isfield(opts.dummies{is},'auc')
				stats.auc_mean(is) = opts.dummies{is}.auc;
				stats.auc_cov(is,:) = 0;
				stats.auc_cov(:,is) = 0;
				%stats.auc_cov(is,is) = max(diag(stats.auc_cov)); % somewhat conservative estimate of the covariance
				stats.auc_cov(is,is) = opts.dummies{is}.auc * (1-opts.dummies{is}.auc) / min(nnz(actuals{1}),nnz(~actuals{1})); % somewhat conservative estimate of the covariance
			end
		end
	end
	
	% Statistical tests
	stats.auc_pvalue = zeros(num_scores);
	for is1 = 1:num_scores
		for is2 = 1:num_scores
			% This corresponds to a contrast matrix L = [1,-1]
			m = stats.auc_mean(is1) - stats.auc_mean(is2);
			v = stats.auc_cov(is1,is1) + stats.auc_cov(is2,is2) - 2*stats.auc_cov(is1,is2);
			stats.auc_pvalue(is1,is2) = normcdf(m/sqrt(v+eps));
		end
	end
	
	% Bootstrap based tests
	rand('seed',123);
	num_bootstraps = opts.num_bootstraps;
	if num_bootstraps > 0
		stats.num_bootstraps = num_bootstraps;
		b_auc  = zeros(num_bootstraps,num_scores);
		b_aupr = zeros(num_bootstraps,num_scores);
		for ib = 1:num_bootstraps
			ir = ceil(rand * num_repetitions);
			sample = ceil(rand(size(actuals{ir})) * numel(actuals{ir}))(:);
			for is = 1:num_scores
				if isfield(opts,'dummies') && isfield(opts.dummies{is},'auc')
					% Dummy values
					b_auc(ib,is) = opts.dummies{is}.auc;
					b_aupr(ib,is) = opts.dummies{is}.aupr;
				else
					b_auc(ib,is)  = calculate_auc(actuals{ir}(sample),scores{is}{ir}(sample));
					b_aupr(ib,is) = calculate_aupr(actuals{ir}(sample),scores{is}{ir}(sample));
				end
			end
		end
		stats.bootstrap_mean_auc  = mean(b_auc);
		stats.bootstrap_mean_aupr = mean(b_aupr);
		stats.bootstrap_std_auc  = std(b_auc);
		stats.bootstrap_std_aupr = std(b_aupr);
		stats.bootstrap_cov_auc  = cov(b_auc);
		stats.bootstrap_cov_aupr = cov(b_aupr);
		for is1 = 1:num_scores
			for is2 = 1:num_scores
				d_auc = b_auc(:,is1) - b_auc(:,is2);
				d_aupr = b_aupr(:,is1) - b_aupr(:,is2);
				stats.bootstrap_z_auc(is1,is2)  = mean(d_auc) / (std(d_auc)+eps);
				stats.bootstrap_z_aupr(is1,is2) = mean(d_aupr) / (std(d_aupr)+eps);
				stats.bootstrap_p_auc(is1,is2)  = mean(d_auc > 0);
				stats.bootstrap_p_aupr(is1,is2) = mean(d_aupr > 0);
			end
		end
	end
end
